#include "injection_pump_global.h"
#include "injection_pump_controller.h"
#include "serial.h"

const char reply_cmd_recv[] = "Command received\n";
const char reply_bad_cmd[] = "Bad command\n";
const char reply_bad_speed[] = "Speed error\n";

static void echo_task(void *arg)
{
    char *data = (char *)malloc(SERIAL_BUFFER_SIZE);
    char *cmd = (char *)malloc(SERIAL_BUFFER_SIZE);
    memset(data, 0, SERIAL_BUFFER_SIZE);
    memset(cmd, 0, SERIAL_BUFFER_SIZE);
    double value0 = 0;
    double value1 = 0;
    while (1)
    {
        int len = uart_read_bytes(SERIAL_PORT_NUM, data, (SERIAL_BUFFER_SIZE - 1), 20 / portTICK_PERIOD_MS);
        if (len > 0)
        {

            int cnt = sscanf(data, "%s%lf%lf", cmd, &value0, &value1);

            uart_write_bytes(SERIAL_PORT_NUM, reply_cmd_recv, sizeof(reply_cmd_recv) - 1);

            double pump_distance = (4 * value0 * 1000) / (PI * PUMP_DIAMETER * PUMP_DIAMETER);
            if (!strcmp(cmd, "push") && cnt == 2)
            {
                injection_pump_run(pump_distance, PUSH);
            }
            else if (!strcmp(cmd, "pull") && cnt == 2)
            {
                injection_pump_run(pump_distance, PULL);
            }
            else if (!strcmp(cmd, "set_speed") && cnt == 3)
            {
                if (value0 <= value1)
                {
                    set_pump_speed(value0 / 1000, value1 / 1000);
                    printf("%d %d",LEDC_LOW_FREQUENCY,LEDC_HIGH_FREQUENCY);
                }
                else
                {
                    uart_write_bytes(SERIAL_PORT_NUM, reply_bad_speed, sizeof(reply_bad_speed) - 1);
                }
            }
            else if (!strcmp(cmd, "set_pump_diameter") && cnt == 2)
            {
                PUMP_DIAMETER = value0;
            }
            else
            {
                uart_write_bytes(SERIAL_PORT_NUM, reply_bad_cmd, sizeof(reply_bad_cmd) - 1);
            }
        }
    }
}

void serial_init_and_start(void)
{

    uart_config_t uart_config = {
        .baud_rate = SERIAL_BAUD_RATE,
        .data_bits = UART_DATA_8_BITS,
        .parity = UART_PARITY_DISABLE,
        .stop_bits = UART_STOP_BITS_1,
        .flow_ctrl = UART_HW_FLOWCTRL_DISABLE,
        .source_clk = UART_SCLK_DEFAULT,
    };

    ESP_ERROR_CHECK(uart_driver_install(SERIAL_PORT_NUM, SERIAL_BUFFER_SIZE, 0, 0, NULL, 0));
    ESP_ERROR_CHECK(uart_param_config(SERIAL_PORT_NUM, &uart_config));
    ESP_ERROR_CHECK(uart_set_pin(SERIAL_PORT_NUM, IO_PIN_SERIAL_TXD, IO_PIN_SERIAL_RXD, SERIAL_RTS, SERIAL_CTS));

    xTaskCreate(echo_task, "serial_task", SERIAL_STACK_SIZE, NULL, 10, NULL);
}
